package com.survey7.flashcards;

/*
Copyright 2012 Sanjay Rao, Seattle WA USA

This file is part of Flash Cards GPL.

Flash Cards GPL is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

Flash Cards GPL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Flash Cards GPL.  If not, see <http://www.gnu.org/licenses/>.
*/

import android.app.Application;
import android.app.Activity;
import android.widget.TextView;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.content.SharedPreferences;
import android.content.Context;
import android.util.Log;
import java.io.FilenameFilter;
import java.io.File;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.widget.ImageView;
import android.text.Html;
import android.os.Vibrator;

public class Globals {
	private static final String DBG="FlashCards";
	private final static String defshared="_DEF";
	BitmapFactory BF=null;
	private Context context;
	public quiz quiz;
	public final int defcountdown=20;
	private boolean isloaded=false;
	private int cardjpg=-1;
	public int white=0xffffffff,black=0xff000000,blue=0xffb5ddec,red=0xffff0000;
	public boolean repeatmissed; // NOT SAVED TO PREFS
	public boolean multiplechoice; // NOT SAVED TO PREFS
	public boolean needscan=false;
	public float dpratio,flingthreshold,nflingthreshold;
	private Vibrator vibe=null;

	public String frontjpeg;
	public int doanimation; // 0=>none, 1=> left/right slides
	public int uimode; // 0=> multiple choice, repeat, 1=> no repeat, 2=> no choice
	public int subset; // 0=> all, 1=> 1,3,5, 2=> 2,4,6
	public int direction; // 0=>randomize,1=>forward,2=>reverse
	public int facts; // 0=>show facts on reverse, 1=>don't
	public int fgcolor;
	public int bgcolor;
	public boolean isgplapproved;
	public int countdown;
	public String librarylist;
	public int lastquiz_int;
	public String lastquiz_filename;
	public String lastquiz_title;
	public boolean showbuiltins;
	public boolean denyvibrate;

	public void cleanquiz() {
		quiz=null;
		System.gc();
	}
	
	private void setvars() {
		cardjpg=-1;
		if (frontjpeg.equals("white")) cardjpg=R.drawable.indexcard;
		else if (frontjpeg.equals("blue")) cardjpg=R.drawable.bluecard;
		switch (uimode) {
			case 0: repeatmissed=multiplechoice=true; break;
			case 1: repeatmissed=false; multiplechoice=true; break;
			case 2: repeatmissed=multiplechoice=false; break;
		}
	}
	public void loadpreferences() {
		SharedPreferences settings=context.getSharedPreferences(defshared,0);
		isgplapproved=settings.getBoolean("gpl",false);
		doanimation=settings.getInt("animate",1);
		countdown=settings.getInt("countdown",defcountdown);
		librarylist=settings.getString("librarylist","");
		frontjpeg=settings.getString("frontjpeg","blue");
		fgcolor=settings.getInt("fgcolor",black);
		bgcolor=settings.getInt("bgcolor",blue);
		uimode=settings.getInt("uimode",2);
		subset=settings.getInt("subset",0);
		direction=settings.getInt("direction",0);
		facts=settings.getInt("facts",0);
		lastquiz_int=settings.getInt("lq_int",1);
		lastquiz_filename=settings.getString("lq_fn","");
		lastquiz_title=settings.getString("lq_title","");
		showbuiltins=settings.getBoolean("showbuiltins",true);
		denyvibrate=settings.getBoolean("denyvibrate",false);
		setvars();
	}
	public void savepreferences() {
		SharedPreferences settings=context.getSharedPreferences(defshared,0);
		SharedPreferences.Editor editor=settings.edit();
		editor.putBoolean("gpl",isgplapproved);
		editor.putInt("animate",doanimation);
		editor.putInt("countdown",countdown);
		editor.putString("librarylist",librarylist);
		editor.putString("frontjpeg",frontjpeg);
		editor.putInt("fgcolor",fgcolor);
		editor.putInt("bgcolor",bgcolor);
		editor.putInt("uimode",uimode);
		editor.putInt("subset",subset);
		editor.putInt("direction",direction);
		editor.putInt("facts",facts);
		editor.putInt("lq_int",lastquiz_int);
		editor.putString("lq_fn",lastquiz_filename);
		editor.putString("lq_title",lastquiz_title);
		editor.putBoolean("showbuiltins",showbuiltins);
		editor.putBoolean("denyvibrate",denyvibrate);
		editor.commit();
		setvars();
	}
	public void settextview(Activity a, int id, String s) {
		TextView tv=(TextView)a.findViewById(id);
		tv.setText(s);
		tv.setTextColor(fgcolor);
	}
	private String htmlesc(String in) {
		String out;
		out=in.replaceAll("&","&amp;");
		out=out.replaceAll("<","&lt;");
		out=out.replaceAll(">","&gt;");
		return in;
	}
	public void settextview3(Activity a, int id, String s, String wikisearch) {
		TextView tv=(TextView)a.findViewById(id);
		if (s.length()==0) {
			tv.setVisibility(View.GONE);
		} else {
			tv.setVisibility(View.VISIBLE);
			if (wikisearch!=null) tv.setText(Html.fromHtml("<a href=\"http://en.wikipedia.org/wiki/Special:Search?search="+htmlesc(wikisearch)+"\">"+htmlesc(s)+"</a>"));
			else tv.setText(s);
			tv.setTextColor(fgcolor);
		}
	}
	public void setimageview2(Activity a, int id, rawimage bmp) {
		ImageView iv=(ImageView)a.findViewById(id);
		if (bmp==null) {
			iv.setVisibility(View.GONE);
		} else {
			iv.setVisibility(View.VISIBLE);
			iv.setImageBitmap(makebitmap(bmp));
		}
	}
	public void setnotion3(Activity a, int idtv, int idiv, String str, rawimage bmp, String wikisearch) {
		settextview3(a,idtv,str,wikisearch);
		setimageview2(a,idiv,bmp);
	}
	public void settextview(Activity a, int id, int sid) {
		settextview(a,id,a.getString(sid));
	}
	public void setbackground(Activity a, int id, boolean isfront) {
		View v=(View)a.findViewById(id);
		if ((cardjpg>=0)&&(isfront)) v.setBackgroundResource(cardjpg);
		else v.setBackgroundColor(bgcolor);
	}
	public void startanimation(Activity a, int id, int aid) {
		if (doanimation!=1) return;
		View v=(View)a.findViewById(id);
		Animation anim=AnimationUtils.loadAnimation(a,aid);
		v.startAnimation(anim);
	}
	public void vibrate(long duration) {
		if (vibe==null) return;
		if (denyvibrate) return;
		vibe.vibrate(duration);
	}
	public void init(Context ctx) {
		context=ctx; 
		if (!isloaded) loadpreferences();
		dpratio=ctx.getResources().getDisplayMetrics().density;
		flingthreshold=150*dpratio;
		nflingthreshold=-1*flingthreshold;
		vibe=(Vibrator)ctx.getSystemService(Context.VIBRATOR_SERVICE);
/*
		API 11
		if (vibe!=null) {
			if (!vibe.hasVibrator()) vibe=null;
		}
*/
		if (BF==null) BF=new BitmapFactory();
	}
	static public String notionjoin(notion[] arr, String join) {
		if (arr.length==0) return "";
		StringBuilder sb=new StringBuilder();
		int i,n;
		n=arr.length;
		sb.append(arr[0].tostring());
		for (i=1;i<n;i++) { sb.append(join); sb.append(arr[i].tostring()); }
		return sb.toString();
	}
	public void startquiz(quiz quiz_in, boolean autostart, boolean quizmissed) {
		if (quiz_in!=null) quiz=quiz_in;
		if (quiz==null) return;
		quiz.autostart=autostart;
		quiz.reset(quizmissed);
		if (!quizmissed) switch (subset) {
			case 1: quiz.quizinterleave(true); break;
			case 2: quiz.quizinterleave(false); break;
		}
		switch (direction) {
			case 0: quiz.randomize(); break;
			case 2: quiz.reverse(); break;
			default: quiz.forward(); break;
		}
	}
	public String loadquiz(int internalindex, String filename) {
		InputStream is;
		switch (internalindex) {
			case 1: is=context.getResources().openRawResource(R.raw.builtin1); break;
			case 2: is=context.getResources().openRawResource(R.raw.builtin2); break;
			case 3: is=context.getResources().openRawResource(R.raw.builtin3); break;
			case 4: is=context.getResources().openRawResource(R.raw.builtin4); break;
			default:
				is=datautils.openfile(filename);
				break;
		}
		if (is==null) return "Error loading data";
		System.gc();
		quiz=buildquiz.load(is);
		System.gc();
		if (quiz==null) return "Error parsing data";
		lastquiz_int=internalindex;
		lastquiz_filename=filename;
		lastquiz_title=quiz.title;
		savepreferences();
		return null;
	}
	public Bitmap makebitmap(rawimage raw) {
		if (raw==null) return null;
		System.gc();
		return BF.decodeByteArray(raw.data,0,raw.data.length);
	}
}
